module Fedi.Routes where

import Web.Twain qualified as Twain
import Web.Twain.Types qualified as Twain
import Data.String (fromString)
import Data.Aeson qualified as A
import Data.ByteString.Lazy qualified as BSL
import Fedi.Types
import Fedi.Activity
import Fedi.Actor
import Fedi.Webfinger

-- * Routes

routes :: UserDetails -> [Twain.Middleware]
routes details =
  [ Twain.get (matchUser details) do
    handleUser details

  , Twain.get (matchOutbox details) do
    handleOutbox details

  , Twain.get matchWebfinger do
    handleWebfinger details
  ]

jsonLD :: BSL.ByteString -> Twain.Response
jsonLD =
  Twain.raw
    Twain.status200
    [(Twain.hContentType, "application/activity+json; charset=utf-8")]

-- * User

matchUser :: UserDetails -> Twain.PathPattern
matchUser details = fromString ("/" <> details.username)

handleUser :: UserDetails -> Twain.ResponderM a
handleUser details = do
  let content = makeActor details
  Twain.send $ jsonLD (A.encode content)

-- * Webfinger

matchWebfinger :: Twain.PathPattern
matchWebfinger = "/.well-known/webfinger"

handleWebfinger :: UserDetails -> Twain.ResponderM b
handleWebfinger details = do
  resource <- Twain.param "resource"
  let webfinger = makeWebfinger details
  if resource == ppSubject webfinger.wfSubject
    then do
      Twain.send $ jsonLD (A.encode webfinger)
    else do
      Twain.next

-- * Outbox

matchOutbox :: UserDetails -> Twain.PathPattern
matchOutbox details =
  fromString ("/" <> details.username <> "/outbox")

handleOutbox :: UserDetails -> Twain.ResponderM b
handleOutbox details = do
  let content = Collection { collectionSummary = details.username <> "'s notes", collectionItems = [] } :: Outbox
  Twain.send $ jsonLD (A.encode content)
